// -*- C++ -*-
// --------------------------------------------------------------------
// The Ipe document.
// --------------------------------------------------------------------
/*

    This file is part of the extensible drawing editor Ipe.
    Copyright (C) 1993-2003  Otfried Cheong

    Ipe is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Ipe is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
    or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
    License for more details.

    You should have received a copy of the GNU General Public License
    along with Ipe; if not, you can find it at
    "http://www.gnu.org/copyleft/gpl.html", or write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef IPEDOC_H
#define IPEDOC_H

#include "ipeobj.h"
#include "ipepage.h"
#include "ipeimage.h"

class IpeStyleSheet;

// --------------------------------------------------------------------

typedef std::vector<IpePage *> IpePageSeq;

class IPE_EXPORT IpeDocument : public IpePageSeq {
public:
  //! Properties of a document
  struct SProperties {
    SProperties();
    IpeString iTitle;
    IpeString iAuthor;
    IpeString iSubject;
    IpeString iKeywords;
    IpeString iPreamble;
    bool iFullScreen;
    bool iCropBox;
    //! Date/time in PDF style "D:20010428191400" format.
    IpeString iCreated;
    IpeString iModified;
    //! Program that created this document (e.g. "Ipe 7.5").
    IpeString iCreator;
  };

  IpeDocument();
  ~IpeDocument();

  static IpeDocument *New(const char *fname);

  bool Save(const char *fname, IpeString creator);
  void SaveAsXml(IpeStream &stream, IpeString creator,
		 bool usePdfBitmaps = false);

  //! Return document properties.
  inline SProperties Properties() const { return iProperties; }
  void SetProperties(const SProperties &info);

  void SetStyleSheet(IpeStyleSheet *sheet);
  //! Return const pointer to topmost stylesheet.
  const IpeStyleSheet *StyleSheet() const { return iStyleSheet; }
  //! Return modifiable stylesheet.
  IpeStyleSheet *GetStyleSheet() { return iStyleSheet; }

  bool IsEdited() const;
  void SetEdited(bool edited);

  bool NotifyStyle(IpeAttributeSeq &seq) const;

  //! Return attribute repository.
  inline IpeRepository *Repository() { return &iRepository; }
  //! Return attribute repository (const version).
  inline const IpeRepository *Repository() const { return &iRepository; }

private:
  IpeDocument(const IpeDocument &rhs);
  IpeDocument &operator=(const IpeDocument &rhs);
protected:
  IpeStyleSheet *iStyleSheet;
  SProperties iProperties;
  bool iEdited;
  IpeRepository iRepository;
};

// --------------------------------------------------------------------
#endif
