// -*- C++ -*-
// --------------------------------------------------------------------
// AppUi
// --------------------------------------------------------------------
/*

    This file is part of the extensible drawing editor Ipe.
    Copyright (C) 1993-2003  Otfried Cheong

    Ipe is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Ipe is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
    or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
    License for more details.

    You should have received a copy of the GNU General Public License
    along with Ipe; if not, you can find it at
    "http://www.gnu.org/copyleft/gpl.html", or write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef APPUI_H
#define APPUI_H

#include "ipebase.h"
#include "ipecolor.h"
#include "ipestyle.h"
#include "ipesnap.h"
#include "ipelet.h"
#include "ipeundo.h"

#include "prefs.h"
#include "canvas.h"
#include "overlay.h"
#include "master.h"
#include "ipepdfdoc.h"

#include <qmainwindow.h>
#include <qscrollview.h>
#include <qpixmap.h>
#include <qtoolbutton.h>
#include <qtranslator.h>
#include <qspinbox.h>

class IpePage;

class QSpinBox;
class QToolBar;
class QPopupMenu;
class QComboBox;
class QTextBrowser;
class QListBox;
class QLineEdit;
class QWidgetStack;
class DecimalSpinBox;
class QPushButton;
class QColor;
class QAction;
class QActionGroup;

// --------------------------------------------------------------------

// in main.cpp
void StartManual(QString url);

class ZoomSpinBox : public QSpinBox {
  Q_OBJECT
public:
  ZoomSpinBox(QWidget *parent = 0);
public slots:
  virtual void stepUp();
  virtual void stepDown();
};

// --------------------------------------------------------------------

class AppUi : public QMainWindow, public OverlayProvider,
	      public ObjectAdder, public IpeletHelper {
  Q_OBJECT

public:
  enum TCommands { ECmdGroup, ECmdUngroup, ECmdDuplicate, ECmdFront, ECmdBack,
		   ECmdCut, ECmdCopy, ECmdPaste, ECmdDelete, ECmdSelectAll,
		   ECmdNewLayer, ECmdRenameLayer, ECmdDeleteLayer,
		   ECmdHideLayer, ECmdDimLayer, ECmdLockLayer, ECmdUnsnapLayer,
		   ECmdSelectLayer, ECmdLineWidth, ECmdDashStyle,
		   ECmdTextSize, ECmdMarkShape, ECmdMarkSize, ECmdUndo,
		   ECmdRedo, ECmdEdit,
		   ECmdSetArrows, ECmdArrowSize, ECmdStroke, ECmdFill,
		   ECmdComposePaths, ECmdJoinPaths, ECmdDecomposePath,
		   ECmdHere, ECmdSetOrigin, ECmdResetOrigin,
		   ECmdSetOriginSnap, ECmdSetDirection, ECmdSetDirectionSnap,
		   ECmdSetLine, ECmdSetLineSnap,
		   ECmdMoveToLayer,
  };

public:
  AppUi(QWidget* parent=0, const char* name=0, WFlags f=0);
  ~AppUi();

  bool Load(QString fileName);
  bool Save(QString fname, bool warnOverwrite);

  static QPixmap ToPng(const IpePdfDocument *doc,
		       const IpePage *page, double zoom);

public slots:
  void Cmd(int cmd);
  void UndoCmd(int cmd);
  void SnapCmd(int cmd);

  void Cut();
  void Copy();
  void Paste();

  void NewWindow();

  void About();
  void AboutQt();

  void NewDoc();
  void Save();
  void SaveAs();
  void SaveAsBitmap();
  void SaveKeys();
  void Load();

  void EditObject();
  void EditDocProps();
  void EditPrefs();
  void Manual();
  void ShowDebugWindow();
  bool RunLatex(int);
  void EditPresentation();

  void CreatePage();
  void DeletePage();
  void CopyPage(int cut);
  void PastePage();

  void NormalSize();
  void FitPage();
  void FitObjects();
  void FitSelection();

  void StyleSheets();

  void FirstPage();
  void LastPage();

  void SetCreator(QAction *action);

  void RunIpelet(int);

protected:
  void closeEvent(QCloseEvent *ev);

private slots:
  void AboutToShowEditMenu();

  void MediaChanged(int);
  void PageChanged(int);
  void ResolutionChanged(int);
  void AbsoluteAttributes();
  void AbsoluteSnapping();

  void SetStrokeColor();
  void SetFillColor();
  void SetStrokeColorName(int item);
  void SetFillColorName(int item);

  void LineWidthChanged(int id);
  void AbsLineWidthChanged(int val);
  void DashStyleChanged(int id);
  void TextSizeChanged(int id);
  void AbsTextSizeChanged(int val);
  void MarkShapeChanged(int id);
  void MarkSizeChanged(int id);
  void AbsMarkSizeChanged(int val);
  void ArrowSizeChanged(int id);
  void AbsArrowSizeChanged(int val);
  void ArrowChanged(int id);
  void GridSizeChanged(int id);
  void AbsGridSizeChanged(int val);
  void AngleSizeChanged(int id);
  void AbsAngleSizeChanged(int val);

  void SnapChanged(bool);
  void GridVisible();
  void LayerChanged(int);

  void PreferencesChanged();

private: // from OverlayProvider
  virtual void RequestOverlay(QMouseEvent *ev);
  virtual void SetDrawingMode(bool mode);
  virtual void WheelZoom(int delta);
  virtual IpeAttribute PageColor();

private: // from ObjectAdder
  virtual void ObjAdd_AddObject(IpeObject *obj);
  virtual const IpeAllAttributes &ObjAdd_Attributes();
  virtual const IpeStyleSheet *ObjAdd_StyleSheet();
  virtual void ObjAdd_AddUndoItem(IpePage *page, QString s);
  virtual void ObjAdd_AddUndoItem(IpePage::iterator it, IpePage *page,
				  const IpePgObject &original, QString s);

private: // from IpeletHelper
  virtual const IpeAllAttributes &Attributes() const;
  virtual int CurrentLayer() const;
  virtual void Message(const char *msg);
  virtual int MessageBox(const char *text, const char *button1,
			 const char *button2, const char *button3);
  virtual bool GetString(const char *prompt, IpeString &str);
  virtual const IpeSnapData &SnapData() const;
  virtual IpeDocument *Document(bool markAllEdited);

private:
#if QT_VERSION >= 300
  QKeySequence Key(const char *src, const char *comment);
#else
  int Key(const char *src, const char *comment);
#endif
  inline IpePage *Page() { return (*iDoc)[iPno]; }
  void FitBox(const IpeRect &box);
  void UpdateLayers();
  void SetMediaFromPage();
  void SwitchAttributeUi();
  void SwitchSnapUi();
  void SetCaption();
  void ShowStyleInUi();
  void AddUndo(IpeUndoItem *item);

  void ErrMsg(QString str);

  double AbsValue(IpeAttribute attr);
  void ConvertAbsolute(DecimalSpinBox *spin, IpeAttribute &attr);

  void ResetCombo(IpeKind kind, QComboBox *combo,
		  IpeAttribute &attr, QPixmap *pixmap = 0);
  void InsertColor(QComboBox *combo, IpeAttribute sym);
  void SetColorFromQColor(IpeAttribute &colorToSet, QColor &qColor,
			  QPushButton *button);

  virtual void keyPressEvent(QKeyEvent *ev);

private:
  enum TMenuIds { EViewMenuId = 1000, ESnapMenuId, EAbsoluteAttributesMenuId,
		  EAbsoluteSnappingMenuId, EGridVisibleMenuId };

  // UI components
  Canvas *iCanvas;

  QPopupMenu *iEditMenu;
  QPopupMenu *iSnapMenu;
  QPopupMenu *iModeMenu;
  QPopupMenu *iViewMenu;
  QPopupMenu *iLayerMenu;
  QPopupMenu *iIpeletMenu;

  QToolBar *iFileTools;
  QToolBar *iResolutionTools;
  QToolBar *iObjectTools;
  QToolBar *iPageTools;
  QToolBar *iLineTools;
  QToolBar *iSizeTools;
  QToolBar *iColorTools;
  QToolBar *iSnapTools;
  QToolBar *iLayerTools;

  QWidgetStack *iStrokeColorStack;
  QWidgetStack *iFillColorStack;
  QWidgetStack *iLineWidthStack;
  QWidgetStack *iArrowSizeStack;
  QWidgetStack *iTextSizeStack;
  QWidgetStack *iMarkSizeStack;
  QWidgetStack *iGridSizeStack;
  QWidgetStack *iAngleSizeStack;

  QSpinBox *iResolution;

  QSpinBox *iPageNumber;
  QComboBox *iMediaSize;

  QComboBox *iStrokeColor;
  QPushButton *iAbsStrokeColor;
  QComboBox *iFillColor;
  QPushButton *iAbsFillColor;

  QComboBox *iLineWidth;
  DecimalSpinBox *iAbsLineWidth;
  QComboBox *iDashStyle;
  QComboBox *iArrow;
  QComboBox *iArrowSize;
  DecimalSpinBox *iAbsArrowSize;

  QActionGroup *iModeActionGroup;
  QAction *iModeAction[OverlayFactory::Num];
  QComboBox *iMarkShape;
  QComboBox *iMarkSize;
  DecimalSpinBox *iAbsMarkSize;
  QComboBox *iTextSize;
  DecimalSpinBox *iAbsTextSize;

  QActionGroup *iSnapActionGroup;
  QAction *iSnapAction[6];
  QComboBox *iGridSize;
  DecimalSpinBox *iAbsGridSize;
  QComboBox *iAngleSize;
  DecimalSpinBox *iAbsAngleSize;

  QListBox *iLayerList;

  // Information about the document

  //! The document's filename (null if not yet saved).
  QString iFileName;
  //! The document.
  IpePdfDocument *iDoc;
  //! The currently displayed page no.
  int iPno;
  //! The current layer.
  int iLayer;
  //! Whether latex needs to be run.
  bool iNeedLatex;
  //! Displaying attributes absolute (or symbolic)
  bool iAbsoluteAttributes;
  //! Displaying absolute snap sizes.
  bool iAbsoluteSnapping;
  //! The attributes set in the UI.
  IpeAllAttributes iAttributes;
  //! Snapping state
  IpeSnapData iSnapData;
  //! Current Ipe mode.
  int iMode;
  //! All symbolic attributes in style sheet
  IpeAttributeSeq iSyms[IpeAttribute::ETemplate + 1];

  //! Undo stack
  IpeUndoStack iUndo;
  // Ipelets
  //! Ipelet repository
  IpeletMaster iIpeletMaster;
  bool iIpeletNoUndo;

  //! Configurable key sequences
  QTranslator iKeyTranslator;
};

// --------------------------------------------------------------------
#endif
