// -*- C++ -*-
// --------------------------------------------------------------------
// Overlay for creating IpePath objects
// --------------------------------------------------------------------
/*

    This file is part of the extensible drawing editor Ipe.
    Copyright (C) 1993-2003  Otfried Cheong

    Ipe is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Ipe is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
    or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
    License for more details.

    You should have received a copy of the GNU General Public License
    along with Ipe; if not, you can find it at
    "http://www.gnu.org/copyleft/gpl.html", or write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef CREATEPATH_H
#define CREATEPATH_H

#include "overlay.h"

// --------------------------------------------------------------------

class IpeSegmentSubPath;

class CreateRectangle : public Overlay {
 public:
  CreateRectangle(QMouseEvent *ev, Canvas *canvas, ObjectAdder *objectAdder);

  virtual void Draw(QPaintEvent *ev, QPainter *painter) const;
  virtual void MousePress(QMouseEvent *ev);
  virtual void MouseMove(QMouseEvent *ev);

private:
  void Compute();

private:
  ObjectAdder *iObjectAdder;
  bool iSquare;
  IpeVector iV[2];
};

class CreateSplinegon : public Overlay {
 public:
  CreateSplinegon(QMouseEvent *ev, Canvas *canvas, ObjectAdder *objectAdder);

  virtual void Draw(QPaintEvent *ev, QPainter *painter) const;
  virtual void MousePress(QMouseEvent *ev);
  virtual void MouseMove(QMouseEvent *ev);
  virtual void KeyPress(QKeyEvent *ev);
  void Explain() const;

 private:
  ObjectAdder *iObjectAdder;
  std::vector<IpeVector> iV;
};

class CreatePath : public Overlay {
 public:
  CreatePath(QMouseEvent *ev, Canvas *canvas, ObjectAdder *objectAdder,
	     bool closed, bool spline);
  virtual ~CreatePath();

  virtual void Draw(QPaintEvent *ev, QPainter *painter) const;
  virtual void MousePress(QMouseEvent *ev);
  virtual void MouseMove(QMouseEvent *ev);
  virtual void KeyPress(QKeyEvent *ev);

 private:
  static IpeMatrix ComputeMatrix(const IpeVector &v0, const IpeVector &v1,
				 const IpeVector &v2);

  bool Decrease(int k, std::vector<IpeVector> &v);
  void RecomputeMatrix();
  void Update();

 private:
  ObjectAdder *iObjectAdder;
  IpeSegmentSubPath *iSP;
};

// --------------------------------------------------------------------
#endif
