// -*- C++ -*-
// --------------------------------------------------------------------
// Overlay for creating IpeText objects
// --------------------------------------------------------------------
/*

    This file is part of the extensible drawing editor Ipe.
    Copyright (C) 1993-2003  Otfried Cheong

    Ipe is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Ipe is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
    or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
    License for more details.

    You should have received a copy of the GNU General Public License
    along with Ipe; if not, you can find it at
    "http://www.gnu.org/copyleft/gpl.html", or write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef CREATETEXT_H
#define CREATETEXT_H

#include "overlay.h"
#include "ipetext.h"
#include "dlgtext.h"

// --------------------------------------------------------------------

class DialogCreateText : public DlgCreateText {
  Q_OBJECT
public:
  DialogCreateText(QWidget* parent, QString caption,
		   QString label, QString &data);
public slots:
  void UpdateData();
private:
  QString &iData;
};

// --------------------------------------------------------------------

class CreateText : public Overlay {
 public:
  enum TMode { ELabel, EMath, EMinipage };

  static void New(QMouseEvent *ev, Canvas *canvas, ObjectAdder *objectAdder,
		  TMode mode);
  static bool Edit(IpeText *obj, const IpeStyleSheet *sheet);

  CreateText(QMouseEvent *ev, Canvas *canvas, ObjectAdder *objectAdder);

  virtual void Draw(QPaintEvent *ev, QPainter *painter) const;
  virtual void MousePress(QMouseEvent *ev);
  virtual void MouseMove(QMouseEvent *ev);

 private:
  ObjectAdder *iObjectAdder;
  IpeVector iV[2];
};

// --------------------------------------------------------------------

class TextWidthChanger : public Overlay {
 public:
  TextWidthChanger(Canvas *canvas, IpeText *text);

  virtual void Draw(QPaintEvent *ev, QPainter *painter) const;
  virtual void MousePress(QMouseEvent *ev);
  virtual void MouseRelease(QMouseEvent *ev);
  virtual void MouseMove(QMouseEvent *ev);

 private:
  IpeText *iObj;
  IpeVector iV[2];
  bool iDragging;
  IpeVector iMouseDown;
};

// --------------------------------------------------------------------
#endif
