// -*- C++ -*-
// --------------------------------------------------------------------
// Dialogs
// --------------------------------------------------------------------
/*

    This file is part of the extensible drawing editor Ipe.
    Copyright (C) 1993-2003  Otfried Cheong

    Ipe is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Ipe is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
    or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
    License for more details.

    You should have received a copy of the GNU General Public License
    along with Ipe; if not, you can find it at
    "http://www.gnu.org/copyleft/gpl.html", or write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef DIALOGS_H
#define DIALOGS_H

#include "dlgdocprop.h"
#include "dlgpref.h"
#include "dlgdirprefs.h"
#include "dlgpagepres.h"
#include "dlglatexerr.h"

#include "ipedoc.h"
#include "prefs.h"
#include "ipepage.h"

#include <qstringlist.h>

// --------------------------------------------------------------------

class DialogDocumentProperties : public DlgDocumentProperties {
  Q_OBJECT
public:
  DialogDocumentProperties(QWidget* parent, IpeDocument::SProperties &data);
public slots:
  void UpdateData();
  void ShowHelp();
private:
  IpeDocument::SProperties &iData;
};

// --------------------------------------------------------------------

class DialogDirPreferences : public DlgDirectories {
  Q_OBJECT

public:
  DialogDirPreferences(QWidget* parent, SPreferences &data);

public slots:
  void ShowHelp();

private:
  SPreferences &iData;
};

// --------------------------------------------------------------------

class DialogPreferences : public DlgPreferences {
  Q_OBJECT

public:
  DialogPreferences(QWidget* parent, SPreferences &data);

public slots:
  void ShowHelp();
  void UpdateData();
  void ShowDirectories();
  void SetFont();
  void SetTextFont();

private:
  SPreferences &iData;
};

// --------------------------------------------------------------------

class DialogPagePresentation : public DlgPagePresentation {
  Q_OBJECT
public:
  DialogPagePresentation(QWidget* parent, QStringList layers,
			 IpePresentationSeq &data);

public slots:
  virtual void AddNewView();
  virtual void MoveViewUp();
  virtual void RemoveView();
  virtual void ShowHelp();
  virtual void MoveViewDown();
  virtual void EffectChanged(int effectNo);
  virtual void LayerHighlighted();
  virtual void CurrentViewChanged(QListBoxItem *);
  virtual void TransitionChanged(int);
  virtual void DurationChanged(int);

private:
  void UpdateFields();
  void UpdateViewLine(int itemno);
  void UpdateViewState();

private:
  QStringList iLayers;
  IpePresentationSeq &iData;
  int iCurView;
  bool iHaveViews;
  bool iInternal;
};

// --------------------------------------------------------------------

class DialogLatexError : public DlgLatexErr {
  Q_OBJECT
public:
  DialogLatexError(QWidget* parent, QString log);
};

// --------------------------------------------------------------------
#endif
