// --------------------------------------------------------------------
// Ipelet maintaining and calling
// --------------------------------------------------------------------
/*

    This file is part of the extensible drawing editor Ipe.
    Copyright (C) 1993-2003  Otfried Cheong

    Ipe is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Ipe is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
    or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
    License for more details.

    You should have received a copy of the GNU General Public License
    along with Ipe; if not, you can find it at
    "http://www.gnu.org/copyleft/gpl.html", or write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "master.h"
#include "prefs.h"
#include "qipe.h"

#ifdef WIN32
#include <windows.h>
#else
#include <dlfcn.h>
#endif

#include <qdir.h>

#include "debugwin.h"

typedef Ipelet *(*PNewIpeletFn)();

// --------------------------------------------------------------------

/*! \class IpeletMaster
  \brief Maintains the set of Ipelets.
*/

//! Load all Ipelets.
IpeletMaster::IpeletMaster()
{
  QStringList dirs = SPreferences::Static()->iIpeletPath;
  for (int i = 0; i < dirs.count(); ++i) {
    QDir dir(dirs[i]);
    dir.setFilter(QDir::Files);
    dir.setNameFilter("*.dll;*.so*;*.dylib");
    DBG << "Directory '" << IpeQ(dir.path()) << "'\n";
    // if directory does not exist, simply skip it
    if (!dir.isReadable())
      continue;
    QStringList dlls = dir.entryList();
    for (int j = 0; j < dlls.count(); ++j) {
      QString fn = dir.filePath(dlls[j]);
      QFileInfo finfo(fn);
      if (finfo.isSymLink()) {
	DBG << "File '" << IpeQ(fn) << "' is symlink, skipped\n";
	continue;
      }
      DBG << "Loading dll '" << IpeQ(fn) << "'\n";
      PNewIpeletFn pIpelet = 0;
#ifdef WIN32
      HMODULE hMod =
	LoadLibrary(QFile::encodeName(QDir::convertSeparators(fn)));
      if (hMod)
	pIpelet = (PNewIpeletFn) GetProcAddress(hMod, "NewIpelet");
#else
      void *handle = dlopen(QFile::encodeName(fn), RTLD_NOW);
      if (handle)
	pIpelet = (PNewIpeletFn) dlsym(handle, "NewIpelet");
#endif
      if (pIpelet) {
	Ipelet *ipelet = pIpelet();
	assert(ipelet != 0);
	DBG << "Success, Ipelet '" << ipelet->Label() << "'\n";
	push_back(ipelet);
      }
    }
  }
}

//! Destroys all Ipelets.
IpeletMaster::~IpeletMaster()
{
  for (iterator it = begin(); it != end(); ++it)
    delete (*it);
}

// --------------------------------------------------------------------
