// -*- C++ -*-
// --------------------------------------------------------------------
// Latex source to PDF converter
// --------------------------------------------------------------------
/*

    This file is part of the extensible drawing editor Ipe.
    Copyright (C) 1993-2003  Otfried Cheong

    Ipe is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Ipe is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
    or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
    License for more details.

    You should have received a copy of the GNU General Public License
    along with Ipe; if not, you can find it at
    "http://www.gnu.org/copyleft/gpl.html", or write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef PDFLATEX_H
#define PDFLATEX_H

#include "ipepage.h"
#include "ipetext.h"

#include <qstring.h>

class PDFDoc;
class PdfObject;
class PdfFontPool;
class IpeStyleSheet;
class PdfOutputDev;
class Gfx;

// --------------------------------------------------------------------

class PdfLatex {
public:
  PdfLatex();
  ~PdfLatex();

  int ScanPage(IpePage *page);
  int CreateLatexSource(QString preamble, const IpeStyleSheet *sheet,
			QString texFile);
  bool ReadPdf(QString pdfFile);
  bool UpdateTextObjects();
  PdfFontPool *TakeFontPool();

private:
  bool GetXForm(PdfObject &xform);
  bool GetEmbeddedFont(int fno, int objno);
  void Warn(IpeString msg);

private:
  struct SText {
    const IpeText *iText;
    IpeAttribute iSize;
  };

  typedef std::list<SText> TextList;
  typedef std::list<IpeText::XForm *> XFormList;

  PDFDoc *iDoc;

  //! List of text objects scanned. Objects not owned.
  TextList iTextObjects;

  //! List of XForm objects read from PDF file.  Objects owned!
  XFormList iXForms;

  //! The embedded fonts. Owned!
  PdfFontPool *iFontPool;

  //! Maps /F<n> to object number;
  std::map<int, int> iFontObjects;

  friend class TextCollectingVisitor;
};

// --------------------------------------------------------------------
#endif
