// --------------------------------------------------------------------
// Ipelets
// --------------------------------------------------------------------
/*

    This file is part of the extensible drawing editor Ipe.
    Copyright (C) 1993-2003  Otfried Cheong

    Ipe is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Ipe is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
    or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
    License for more details.

    You should have received a copy of the GNU General Public License
    along with Ipe; if not, you can find it at
    "http://www.gnu.org/copyleft/gpl.html", or write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "ipelet.h"

// --------------------------------------------------------------------

//! Pure virtual destructor.
Ipelet::~Ipelet()
{
  // nothing
}

//! Return the number of functions implemented by this Ipelet.
/*! The default implementation returns 1. */
int Ipelet::NumFunctions() const
{
  return 1;
}

//! Return a label for each function implemented by this Ipelet.
/*! This is not called if NumFunctions returns 1. */
const char *Ipelet::SubLabel(int function) const
{
  return 0;
}

//! Return a key sequence to be used as keyboard shortcut for this ipelet.
/*! Null if no shortcut.  The default implementation returns null. */
const char *Ipelet::KeySequence(int function) const
{
  return 0;
}

//! Is it okay to call this Ipelet if the current layer is locked.
/*! It is illegal to insert objects into the current layer if that
  layer is locked.  Many ipelets do indeed create objects in the
  current layer, but it's easy for an ipelet author to forget checking
  whether the layer is locked.  This member function turns around the
  responsibility: when the current layer is locked, Ipe will only call
  an ipelet if LockedLayerOkay() returns \c true.  The default
  implementation returns \c false, so ipelets have to deliberately
  override it if they want to permit being called on a locked
  layer. */
bool Ipelet::LockedLayerOkay(int function) const
{
  return false;
}

// --------------------------------------------------------------------
